from tkinter import *
import time
import copy, math
diskCtab=((0xFF,0x77,0xFF), (0x00,0xFF,0xFF), (0xCC,0xCC,0xCC), (0x00,0x00,0xFF),
          (0xFF,0x55,0x55), (0xFF,0xFF,0x00), (0x77,0x77,0xFF), (0xFF,0x00,0x77),
          (0x77,0xFF,0x77), (0xFF,0x00,0xFF))
#■色クラス
class Color:
    def __init__(self,CList):
        self.R=CList[0]
        self.G=CList[1]
        self.B=CList[2]
    def Bright(self, BR):           #色を明るさで調整
        if   BR > 1 : BR=1
        elif BR < 0 : BR=0
        R = int(BR*self.R); G= int(BR*self.G);B= int(BR*self.B)
        return Color([copy.copy(R),copy.copy(G),copy.copy(B)])
    def toString(self):             #Python用色表現
        return "#%02X%02X%02X" % (self.R, self.G, self.B)
#■円盤クラス
class Disk:
    def __init__(self, Canvas, No, R, TH, Color):
        self.canvas=canvas
        self.X = R                  # X座標は円盤の中央を基準座標とする
        self.Y = TH                 # Y座標は円盤の下を基準座標とする
        self.No = No                # 円盤の番号(最も大きい円盤を0とすること)
        self.R  = R                 # 円盤の半径
        self.TH = TH                # 円盤の厚さ
        self.Color=Color            # 円盤の色
        self.TAG="D%04d" % self.No  # 円盤のタグ
        self.drawDisk()
    def move(self, X, Y):           #円盤を絶対座標で動かす(Pythonでは相対座標で動かす)
        self.canvas.move(self.TAG, X-self.X,Y-self.Y)
        self.X=X; self.Y=Y
    def drawDisk(self):             #グラデーション付きの四角形を描く
        DR = self.R; RR =DR*DR;
        for X in range(2*self.R+1): 
            XX=X-self.R; B=math.sqrt(float(RR-XX*XX))/self.R
            self.canvas.create_line(X,0,X, self.TH,
                                    fill=self.Color.Bright(B).toString(), tag=self.TAG)
        self.canvas.create_rectangle(0, 0, self.R*2,self.TH, fill="", outline="#000000", tag=self.TAG)
#■柱表示(円盤クラスを借用)
def drawPole(canvas):
    Disk(canvas,5001,4,100,Color([255,127,0])).move(100,155)
    Disk(canvas,5002,4,100,Color([255,127,0])).move(250,155)
    Disk(canvas,5003,4,100,Color([255,127,0])).move(400,155)
#■床表示
def drawFloar(canvas):
    canvas.create_rectangle(20, 150, 470,170, fill="#CC4422", outline="#000000", tag="Floar")
#■円盤をすべて表示(初期状態)
def drawDiskAll(canvas,N,TH,DX):
    global DT,DiskDT
    DiskDT=[0 for i in range(N+1)]
    DT=[[0 for i in range(N+1)] for j in range(3)]
    DT[0][0]=copy.copy(N)
    for i in range(N):
        DT[0][i+1]=i
        DiskDT[i]=Disk(canvas,i,50-i*DX,TH,Color(diskCtab[i]))
        DiskDT[i].move(100,150-i*TH)
#■円盤を動かす
def moveFromTo(i,j):
        global DT,DiskDT,TH
        N1=DT[i][0]
        if N1==0:
            print("ポール　%d に円盤がありません %d → %d" % (i, i, j))
        else:
            N2 = DT[j][0] + 1; DiskNo=DT[i][N1]#移動の主処理
            DT[j][N2] = DiskNo
            DT[j][0] = N2; DT[i][0]=N1-1
            #以下、移動アニメーション処理
            X1=int(100+150*i); X2=int(100+150*j); X=X1; DX=1
            if X2<X1: DX=-1
            DiskDT[DiskNo].move(X1,80);root.update();time.sleep(0.03)#上に引き上げる
            DiskDT[DiskNo].move(X1,70);root.update();time.sleep(0.02)
            DiskDT[DiskNo].move(X1,60);root.update();time.sleep(0.01)
            while X!=X2:#目標柱位置に移動
                DiskDT[DiskNo].move(X,50)
                root.update()
                time.sleep(0.001)
                X+=DX
            DiskDT[DiskNo].move(X2,60);root.update();time.sleep(0.03)#下に引き下げる
            DiskDT[DiskNo].move(X2,70);root.update();time.sleep(0.02)
            DiskDT[DiskNo].move(X2,80);root.update();time.sleep(0.01)
            DiskDT[DiskNo].move(X2,150-(N2-1)*TH)#最終位置
#■ハノイの塔の主処理
def hanoi(n,x,y):
    if n==0:return
    m=3-x-y
    hanoi(n-1, x, m)
    moveFromTo(x,y)
    hanoi(n-1, m,y)
#■メイン処理
root=Tk()
root.resizable(0,0)
root.title("ハノイの塔")
root.wm_attributes('-topmost')
canvas=Canvas(root,width=500,height=200, bd=0,highlightthickness=0)
canvas.pack()
drawPole(canvas); drawFloar(canvas)#拝啓表示
N=10                    # 円盤の数
TH=int(80/N)            # 円盤の厚さ
DX=int(45/N)            # 円盤の半径の差(番号が大きくなるにつれて小さくなる)
drawDiskAll(canvas,N,TH,DX)
hanoi(N,0,2)
root.update()
